package com.sams.learnweblogic7.servlets;

//import the utils package
import com.sams.learnweblogic7.utils.*;

//importing standard servlet packages
import javax.servlet.*;
import javax.servlet.http.*;

//import io,util and math packages
import java.io.*;
import java.util.*;
import java.math.*;

/** Shopping cart servlet using cookies
 * 
 * @author Mandar Chitnis
 * @author Lakshmi Ananthamurthy
 * @version 1.1
 */

public class BookShoppingServlet extends HttpServlet
{
	//declaring a global hashtable
		 
	Book book1;
	Book book2;
 	Book book3;

	PrintWriter out;

	private static final int DEFAULT_ZERO_VALUE = 0;
	private static final String EXAMPLE_TYPE = "Shopping Cart Using Cookies";

/** 
*  This method obtains the names of the books and their prices from the        
*  initialization parameters, which are set in the web.xml file
*  It then creates book objects which persist for the life of the servlet.
* @param config A servlet configuration object used by a servlet  
*     container used to pass information to a servlet during initialization
* @exception ServletException Defines a general exception a servlet 
*     can throw when it encounters difficulty.Extends java.lang.exception
*/
	
	public void init(ServletConfig config) throws ServletException
	{
		super.init(config);
		
		//getting the init params
		String book1Id = config.getInitParameter("book1Id");
		String book1Name = config.getInitParameter("book1Name");
		double book1Price = new Double(config.getInitParameter("book1Price")).doubleValue();
		String book1Image = config.getInitParameter("book1Image");
		String book1Description = config.getInitParameter("book1Description");
		book1 = new Book(book1Id, book1Name, book1Price, book1Image, book1Description);

		String book2Id = config.getInitParameter("book2Id");
		String book2Name = config.getInitParameter("book2Name");
		double book2Price = new Double(config.getInitParameter("book2Price")).doubleValue();
		String book2Image = config.getInitParameter("book2Image");
		String book2Description = config.getInitParameter("book2Description");
		book2 = new Book(book2Id, book2Name, book2Price, book2Image, book2Description);


		String book3Id = config.getInitParameter("book3Id");
		String book3Name = config.getInitParameter("book3Name");
		double book3Price = new Double(config.getInitParameter("book3Price")).doubleValue();
		String book3Image = config.getInitParameter("book3Image");
		String book3Description = config.getInitParameter("book3Description");
		book3 = new Book(book3Id, book3Name, book3Price, book3Image, book3Description);

	}//end of init
	
    /** 
         * This method is called when the user enters the path of the servlet in the address      
         *  bar space
         *  or when he clicks on viewcart button.
         * 
         * @exception ServletException Defines a general exception a servlet  
         *     can throw when it encounters difficulty. Extends java.lang.Exception
         * @exception IOException Signals that an I/O exception of some sort  
         *      has occurred. This class is the general class of exceptions produced by 
         *      failed or interrupted I/O operations.
         * @param req The request object passed by the servlet container as an
         *      argument to the servlet's doGet / doPost method
         * @param res The response object is created by the servlet container 
         *      and passed to the servlet's doGet/doPost method
         */
		 
	public void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException{
		doPost(req, res);
	}

	public void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException{
		res.setContentType("text/html");
		out = res.getWriter();
		writeHeaderZone(out , EXAMPLE_TYPE);
		doSessionUsingCookie(out, req, res);	
		writeFooterZone(out);
	}//end of doPost

	public void doSessionUsingCookie(PrintWriter outputObject, HttpServletRequest req, HttpServletResponse res){
		Hashtable shoppingCartHash = null;
		Cookie bookCookie = null;
		Cookie[] cookies = req.getCookies();
		if (cookies==null)
		{
			bookCookie = new Cookie(book1.getBookId(), Integer.toString(DEFAULT_ZERO_VALUE));
			bookCookie.setMaxAge(Integer.MAX_VALUE);
			res.addCookie(bookCookie);

			bookCookie = new Cookie(book2.getBookId(), Integer.toString(DEFAULT_ZERO_VALUE));
			bookCookie.setMaxAge(Integer.MAX_VALUE);
			res.addCookie(bookCookie);
			
			bookCookie = new Cookie(book3.getBookId(), Integer.toString(DEFAULT_ZERO_VALUE));
			bookCookie.setMaxAge(Integer.MAX_VALUE);
			res.addCookie(bookCookie);
		}

		String clickedButton = (req.getParameter("buttonName") == null)?"":req.getParameter("buttonName");

		if(clickedButton.equals("view"))
		{
			outputObject.println("<FORM name=\"viewcart\" action=\"/ShoppingApp/BookShoppingServlet\" method=\"get\">");
			outputObject.println("<B><font face = \"Verdana\" color=\"blue\" size=-2><p align = \"right\">"+EXAMPLE_TYPE+"</p></font></B>");
			int book1Qty = Utility.getDefaultValue(getCookieValue(cookies, book1.getBookId()),DEFAULT_ZERO_VALUE);
			int book2Qty = Utility.getDefaultValue(getCookieValue(cookies, book2.getBookId()),DEFAULT_ZERO_VALUE);
			int book3Qty = Utility.getDefaultValue(getCookieValue(cookies, book3.getBookId()),DEFAULT_ZERO_VALUE);
			writeViewCartZone(outputObject, book1Qty, book2Qty, book3Qty);
			outputObject.println("<p align =\"center\"><INPUT type=\"submit\" name=\"buttonName\" value=\"Empty Shopping Cart\"></p>");
			outputObject.println("</FORM>");
		}
		else if(clickedButton.equals("add") || clickedButton.equals(""))
		{
			updateCookie(req, res, cookies,outputObject);
			outputObject.println("<FORM name=\"addcart\" action=\"/ShoppingApp/BookShoppingServlet\" method=\"post\">");
			outputObject.println("<B><font face = \"Verdana\" color=\"blue\" size=-2><p align = \"right\">"+EXAMPLE_TYPE+"</p></font></B>");
			writeBookListZone(outputObject);
			outputObject.println("<CENTER><TABLE width = 100%><TR>");
			outputObject.println("<TD align = \"center\"><INPUT type=\"submit\" name=\"buttonName\" value=\"add\"></TD>");
			outputObject.println("<TD align = \"center\"><INPUT type=\"submit\" name=\"buttonName\" value=\"view\"></TD>");
			outputObject.println("</TR></TABLE></CENTER>");
			outputObject.println("</FORM>");
		}
		else if(clickedButton.equals("Empty Shopping Cart")){
			//setting the cookie values to null and invalidating the cookies
			weblogic.servlet.security.ServletAuthentication.invalidateAll(req);
			weblogic.servlet.security.ServletAuthentication.killCookie(req);
			Cookie updateBookCookie = null;
			updateBookCookie = new Cookie(book1.getBookId(), Integer.toString(DEFAULT_ZERO_VALUE));
			updateBookCookie.setMaxAge(Integer.MAX_VALUE);
			res.addCookie(updateBookCookie);
			updateBookCookie = new Cookie(book2.getBookId(), Integer.toString(DEFAULT_ZERO_VALUE));
			updateBookCookie.setMaxAge(Integer.MAX_VALUE);
			res.addCookie(updateBookCookie);
			updateBookCookie = new Cookie(book3.getBookId(), Integer.toString(DEFAULT_ZERO_VALUE));
			updateBookCookie.setMaxAge(Integer.MAX_VALUE);
			res.addCookie(updateBookCookie);

			outputObject.println("<FORM name=\"addcart\" action=\"/ShoppingApp/BookShoppingServlet\" method=\"post\">");
			outputObject.println("<B><font face = \"Verdana\" color=\"blue\" size=-2><p align = \"right\">"+EXAMPLE_TYPE+"</p></font></B>");
			writeBookListZone(outputObject);
			outputObject.println("<CENTER><TABLE width = 100%><TR>");
			outputObject.println("<TD align = \"center\"><INPUT type=\"submit\" name=\"buttonName\" value=\"add\"></TD>");
			outputObject.println("<TD align = \"center\"><INPUT type=\"submit\" name=\"buttonName\" value=\"view\"></TD>");
			outputObject.println("</TR></TABLE></CENTER>");
			outputObject.println("</FORM>");
		}
	}

	public void updateCookie(HttpServletRequest req, HttpServletResponse res, Cookie[] bookCookies, PrintWriter outputObject){
		Cookie bookCookie = null;

		int Book1Qty = Utility.getDefaultValue(getCookieValue(bookCookies, book1.getBookId()),DEFAULT_ZERO_VALUE)+Utility.getDefaultValue(req.getParameter("book1_qty"),DEFAULT_ZERO_VALUE);
		int Book2Qty = Utility.getDefaultValue(getCookieValue(bookCookies, book2.getBookId()),DEFAULT_ZERO_VALUE)+Utility.getDefaultValue(req.getParameter("book2_qty"),DEFAULT_ZERO_VALUE);
		int Book3Qty = Utility.getDefaultValue(getCookieValue(bookCookies, book3.getBookId()),DEFAULT_ZERO_VALUE)+Utility.getDefaultValue(req.getParameter("book3_qty"),DEFAULT_ZERO_VALUE);
		
		bookCookie = new Cookie(book1.getBookId(), Integer.toString(Book1Qty));
		bookCookie.setMaxAge(Integer.MAX_VALUE);
		res.addCookie(bookCookie);

		bookCookie = new Cookie(book2.getBookId(), Integer.toString(Book2Qty));
		bookCookie.setMaxAge(Integer.MAX_VALUE);
		res.addCookie(bookCookie);
		
		bookCookie = new Cookie(book3.getBookId(), Integer.toString(Book3Qty));
		bookCookie.setMaxAge(Integer.MAX_VALUE);
		res.addCookie(bookCookie);	
	}

	public String getCookieValue(Cookie[] bookCookies, String cookieName){
		for(int i=0; i<bookCookies.length; i++)
		{
			if((bookCookies[i].getName()).equals(cookieName))
			{
				return bookCookies[i].getValue();
			}
		}
		return null;
	}
	// ***********************************	GENERIC FUNCTIONS	*****************************************//
	public void writeHeaderZone(PrintWriter outputObject , String title){
		outputObject.println("<HTML><TITLE>"+title+"</TITLE>");
		outputObject.println("<BODY><FONT face=\"Arial\" size=\"-1\">");
	}

	public void writeFooterZone(PrintWriter outputObject){
		outputObject.println("</FONT></BODY></HTML>");
	}

	public void writeBookListZone(PrintWriter outputObject){
		outputObject.println("<CENTER><B>Most popular books from Sams Publishing</B></CENTER><BR>");
		outputObject.println("<TABLE width = 100%");
		outputObject.println("<TR><TH>&nbsp;</TH><TH>Book Name and Description</TH><TH>Book Price</TH><TH>No of Books to add to cart</TH></TR>");
		outputObject.println("<TR bgcolor='#cccc99'><TD><img src="+book1.getImagePath()+"></TD><TD align='left'><b>"+book1.getBookName()+"</b><BR>"+book1.getBookDescription()+"</TD><TD align='center'>$"+book1.getBookPrice()+"</TD><TD align='center'><INPUT type = 'text' name='book1_qty' size = 3></TD></TR>");  
		outputObject.println("<TR bgcolor='#eeeeee'><TD><img src="+book2.getImagePath()+"></TD><TD align='left'><b>"+book2.getBookName()+"</b><BR>"+book2.getBookDescription()+"</TD><TD align='center'>$"+book2.getBookPrice()+"</TD><TD align='center'><INPUT type = 'text' name='book2_qty' size = 3></TD></TR>");  
		outputObject.println("<TR bgcolor='#cccc99'><TD><img src="+book3.getImagePath()+"></TD><TD align='left'><b>"+book3.getBookName()+"</b><BR>"+book3.getBookDescription()+"</TD><TD align='center'>$"+book3.getBookPrice()+"</TD><TD align='center'><INPUT type = 'text' name='book3_qty' size = 3></TD></TRf>");  
		outputObject.println("</TD></TR></TABLE><BR>");
	}
	public void writeViewCartZone(PrintWriter outputObject, int book1Qty, int book2Qty, int book3Qty){
		double totBook1Price = Utility.getRoundedAmount(book1Qty * book1.getBookPrice());
		double totBook2Price = Utility.getRoundedAmount(book2Qty * book2.getBookPrice());
		double totBook3Price = Utility.getRoundedAmount(book3Qty * book3.getBookPrice());
		double grandTotal = totBook1Price + totBook2Price + totBook3Price;
		outputObject.println("<TABLE width = 100%>");
		outputObject.println("<CENTER><B>Your Shopping Cart Contains:</B></CENTER><BR>");
		outputObject.println("<TR><TH>&nbsp;</TH><TH>Book Name and description</TH><TH>Book Price</TH><TH>Qty</TH><TH>Total for each book</TH></TR>");
		outputObject.println("<TR bgcolor='#cccc99'><TD><img src="+book1.getImagePath()+"></TD><TD align='left'><b>"+book1.getBookName()+"</b><BR>"+book1.getBookDescription()+"</TD><TD align='left'>$"+book1.getBookPrice()+"</TD><TD align ='center'>"+book1Qty+"</TD><TD align='center'>$"+totBook1Price+"</FONT></TD></TR>");  
		outputObject.println("<TR bgcolor='#eeeeee'><TD><img src="+book2.getImagePath()+"></TD><TD align='left'><b>"+book2.getBookName()+"</b><BR>"+book2.getBookDescription()+"</TD><TD align='left'>$"+book2.getBookPrice()+"</TD><TD align ='center'>"+book2Qty+"</TD><TD align='center'>$"+totBook2Price+"</FONT></TD></TR>");  
		outputObject.println("<TR bgcolor='#cccc99'><TD><img src="+book3.getImagePath()+"></TD><TD align='left'><b>"+book3.getBookName()+"</b><BR>"+book3.getBookDescription()+"</TD><TD align='left'>$"+book3.getBookPrice()+"</TD><TD align ='center'>"+book3Qty+"</TD><TD align='center'>$"+totBook3Price+"</FONT></TD></TR>");  
		outputObject.println("<TR bgcolor='#eeeeee'><TD>&nbsp;</TD><TD>&nbsp;</TD><TD>&nbsp;</TD><TD>&nbsp;</TD><TD align='left'><b>Total = $"+grandTotal+"</b></FONT></TD></TR>");
		outputObject.println("</TD></TR></TABLE>");
	}
/*

	public void doSessionUsingCookies{

	}
*/
}//end of BookShoppingServlet
