package com.sams.learnweblogic7.ejb.stateless;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.InitialContext;
import javax.naming.NamingException;


public class ChefBean implements SessionBean 
{
	private String chefId;
	private static final boolean ENABLE_WEBLOGIC_LOGGING = true;
	private boolean isAvailable;
    private SessionContext ctx;

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Used when create is called on the remote interface
	 */
	public void ejbCreate(){
		webLogicLog("Method ejbCreate has been called");
	}
	
	// Using WebLogic's log service to log events
	private void webLogicLog(String logMessage) {
		if (ENABLE_WEBLOGIC_LOGGING) 
			System.out.println(logMessage);
	} //end of webLogicLog

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Presently not being used
	 */
	public void ejbActivate() {
		webLogicLog("Method ejbActivate has been called");
	}//end of ejbActivate

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Presently not being used
	 */
	public void ejbRemove() {
		webLogicLog("Method ejbRemove has been called");
	}//end of ejbRemove

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Presently not being used
	 */
	public void ejbPassivate() {
		webLogicLog("Method ejbPassivate has been called");
	}//end of ejbPassivate


	/**
	 * Sets the session context.
	 *
	 * @param ctx SessionContext Context for session
	 */
	public void setSessionContext(SessionContext ctx) {
		webLogicLog("Method setSessionContext has been called");
		this.ctx = ctx;
	}//end of setSessionContext


  /**
   * This method is called by the waiter to cook the item.
   *
   * @param orderedItem       MenuItem object ordered item
   * @return                  object CookedItem
   */
	public CookedItemImpl cookItem(MenuItem orderedItem){
		webLogicLog("cook Item called");
		System.out.println("Menuitem is "+orderedItem.getItemDesc());
		CookedItemImpl preparedItem = new CookedItemImpl();
		System.out.println("The cooked Item is:"+orderedItem.getItemDesc());
		if((orderedItem.getItemDesc()).equals("Apple Pie")){
			preparedItem.setItemName("Apple Pie");
			preparedItem.setServingSize("Standard");
			preparedItem.setTaste("Sweet");
			preparedItem.setIngredients("Apple, Sugar, All Purpose Flour, Eggs, Butter");
			preparedItem.setTemperature("Hot");
		}
		else if((orderedItem.getItemDesc()).equals("Banana Split Icecream")){
			preparedItem.setItemName("Banana Split Icecream");
			preparedItem.setServingSize("One Scoop");
			preparedItem.setTaste("Sweet");
			preparedItem.setIngredients("Milk, Sugar, All Purpose Flour, Eggs, Cream, Bananas");
			preparedItem.setTemperature("Cold");
		}
		else if((orderedItem.getItemDesc()).equals("Cheese Pizza")){
			preparedItem.setItemName("Cheese Pizza");
			preparedItem.setServingSize("Standard");
			preparedItem.setTaste("Salty");
			preparedItem.setIngredients("All Purpose Flour, Cheese, Jalapenos, Tomatoes, Bell Peppers, Olives, Salt");
			preparedItem.setTemperature("Hot");
		}
		else if((orderedItem.getItemDesc()).equals("Curly Fries")){
			preparedItem.setItemName("Curly Fries");
			preparedItem.setServingSize("Standard");
			preparedItem.setTaste("Salty");
			preparedItem.setIngredients("Low Cholesterol Oil, Potatoes cut, Salt");
			preparedItem.setTemperature("Hot");
		}
		else if((orderedItem.getItemDesc()).equals("Delhi Kababs")){
			preparedItem.setItemName("Delhi Kababs");
			preparedItem.setServingSize("10 pieces");
			preparedItem.setTaste("Spicy");
			preparedItem.setIngredients("Onions,Coriander powder, Poppy Seeds, Ginger, Salad Oil, Salt, Garlic, Spices, Red Chillies, Chicken, Yogurt");
			preparedItem.setTemperature("Hot");
		}
		else if((orderedItem.getItemDesc()).equals("Fresh Pineapple Cake")){
			preparedItem.setItemName("Fresh Pineapple Cake");
			preparedItem.setServingSize("1 Slice");
			preparedItem.setTaste("Sweet");
			preparedItem.setIngredients("Pineapple, Sugar, All Purpose Flour, Eggs, Butter");
			preparedItem.setTemperature("Room Temperature");
		}
		else if((orderedItem.getItemDesc()).equals("French Fries - Tub")){
			preparedItem.setItemName("French Fries - Tub");
			preparedItem.setServingSize("Large");
			preparedItem.setTaste("Salty");
			preparedItem.setIngredients("Low Cholesterol Oil, Potatoes cut, Salt");
			preparedItem.setTemperature("Hot");
		}
		else if((orderedItem.getItemDesc()).equals("Hot Dog")){
			preparedItem.setItemName("Hot Dog");
			preparedItem.setServingSize("Standard");
			preparedItem.setTaste("Salt");
			preparedItem.setIngredients("Hot Dog Buns, Meat, Bread Crumbs, Oatmeal flour, Little Egg White, Spices - Onion, garlic, salt, pepper etc");
			preparedItem.setTemperature("Hot");
		}
		else if((orderedItem.getItemDesc()).equals("Hamburger")){
			preparedItem.setItemName("Hamburger");
			preparedItem.setServingSize("Standard");
			preparedItem.setTaste("Salty");
			preparedItem.setIngredients("Hamburger, Lettuce, Onions, Tobasco sauce, Cheddar cheese, Kaiser style buns, Pam Cooking Spray, Worcestershire sauce");
			preparedItem.setTemperature("Hot");
		}
		return preparedItem;
	}//end of cookItem
}
