package com.sams.learnweblogic7.ejb.cmp;

import javax.naming.*;
import javax.ejb.*;
import java.util.*;
import javax.rmi.PortableRemoteObject;
import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.transaction.*;


public class ChefBean implements SessionBean 
{
	private String chefId;
	private static final boolean ENABLE_WEBLOGIC_LOGGING = true;
	private boolean isAvailable;
    private SessionContext ctx;

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Used when create is called on the remote interface
	 */
	public void ejbCreate(){
		webLogicLog("Method ejbCreate has been called");
	}
	
	// Using WebLogic's log service to log events
	private void webLogicLog(String logMessage) {
		if (ENABLE_WEBLOGIC_LOGGING) 
			System.out.println(logMessage);
	} //end of webLogicLog

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Presently not being used
	 */
	public void ejbActivate() {
		webLogicLog("Method ejbActivate has been called");
	}//end of ejbActivate

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Presently not being used
	 */
	public void ejbRemove() {
		webLogicLog("Method ejbRemove has been called");
	}//end of ejbRemove

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Presently not being used
	 */
	public void ejbPassivate() {
		webLogicLog("Method ejbPassivate has been called");
	}//end of ejbPassivate


	/**
	 * Sets the session context.
	 *
	 * @param ctx SessionContext Context for session
	 */
	public void setSessionContext(SessionContext ctx) {
		webLogicLog("Method setSessionContext has been called");
		this.ctx = ctx;
	}//end of setSessionContext

	/**
	   * Look up the bean's home interface using JNDI.
	   */
	  private ItemHome lookupHome()
		throws NamingException
	  {
		Hashtable h = new Hashtable();
		h.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
		h.put(Context.PROVIDER_URL, "t3://localhost:7001");
		Context ctx = new InitialContext(h);
		System.out.println("Initial context to look up CMP ItemHome created "+ctx);
		try {
		  Object home = (ItemHome)ctx.lookup("CMP_ItemHome");
		  return (ItemHome) PortableRemoteObject.narrow(home, ItemHome.class);

		} catch (NamingException ne) {
		  webLogicLog("Unable to lookup the EJBHome.  Verify the ejb JNDI name CMP_ItemHome on the WebLogic server");
		  throw ne;
		}
	  }

	/**
	   * Look up the bean's home interface using JNDI.
	   */
	  private ItemLogHome lookupItemLogHome()
		throws NamingException
	  {
		Hashtable h = new Hashtable();
		h.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
		h.put(Context.PROVIDER_URL, "t3://localhost:7001");
		Context ctx = new InitialContext(h);
		System.out.println("Initial context to look up CMP ItemHome created "+ctx);
		try {
		  Object home = (ItemLogHome)ctx.lookup("CMP_ItemLogHome");
		  return (ItemLogHome) PortableRemoteObject.narrow(home, ItemLogHome.class);

		} catch (NamingException ne) {
		  webLogicLog("Unable to lookup the EJBHome.  Verify the ejb JNDI name CMP_ItemLogHome on the WebLogic server");
		  throw ne;
		}
	  }

  /**
   * This method is called by the waiter to cook the item.
   *
   * @param orderedItem       MenuItem object ordered item
   * @return                  object CookedItem
   */
	public CookedItemImpl cookItem(Order orderedItem) throws Exception
	{
		ItemHome home;
		ItemLogHome logHome;
		ItemPK orderedItemPk;
		String EJBQLIngredients;
		String orderedItemName;
		String orderedItemServingSize;
		String orderedItemTaste;
		String orderedItemIngredients;
		String orderedItemTemperature;
		UserTransaction userTx = null;
		webLogicLog("cook Item called");
		int orderedItemId = orderedItem.getItem().getItemId();
		int orderQuantity = orderedItem.getQty();
		CookedItemImpl preparedItem = new CookedItemImpl();
		System.out.println("The ordered item desc is:"+orderedItem.getItem().getItemDesc());
		//call the entity bean now to get the attributes for the cooked Item.
		try{

			// ********************* STARTING THE TRANSACTION ***************************
			userTx = ctx.getUserTransaction();//("javax.transaction.UserTransaction");
			//(javax.transaction.UserTransaction)
			userTx.begin();
			System.out.println(" Bean Managed Transaction STARTED .........");
			// For demonstraation purpose - lets assume that we first put the 'item ordered' and 'time' in to 
			// a temporary table called "itemlog". all the items ordered everyday are entered in this table.
			// This table is used by the resturant manager to find out what items are in demand and how many
			// of each items are sold on the daily basis. inshort it gives total number of all the items 
			// that were sold everyday.
			logHome = lookupItemLogHome();
			logHome.create(orderedItemId , new java.sql.Date(System.currentTimeMillis ()));

			home = lookupHome();
			orderedItemPk = new ItemPK(orderedItemId);
			ItemInterface orderedItemInterface = (ItemInterface)home.findByPrimaryKey(orderedItemPk);		
			int remainingQty = orderedItemInterface.getItemQuantity();
			orderedItemName = orderedItem.getItem().getItemDesc();
			if ((remainingQty - orderQuantity)<=0)
			{
				throw new Exception("Item "+orderedItemName+" Out of Stock");
			}
			preparedItem.setItemName(orderedItemName); 
			orderedItemServingSize = orderedItemInterface.getItemServingSize();
			preparedItem.setServingSize(orderedItemServingSize);
			orderedItemTaste = orderedItemInterface.getItemTaste();
			preparedItem.setTaste(orderedItemTaste);
			orderedItemIngredients = orderedItemInterface.getItemIngredients();
			preparedItem.setIngredients(orderedItemIngredients);
			orderedItemTemperature = orderedItemInterface.getItemTemperature();
			preparedItem.setTemperature(orderedItemTemperature);
			System.out.println("Demonstration of findIngredientsByItemName:");
			ItemInterface demonstrateEJBQL = (ItemInterface)home.findIngredientsByItemName(orderedItemName);
			EJBQLIngredients = demonstrateEJBQL.getItemIngredients();
			// ********************* COMMIT THE TRANSACTION ***************************
			userTx.commit();
			System.out.println(" Bean Managed Transaction COMMITED .........");
			System.out.println("Item ingredients obtained :");
			System.out.println();
		}
		catch(Exception e){
			try
			{
				// ********************* ROLLBACK THE TRANSACTION ***************************
				userTx.rollback();
			   System.out.println(" Bean Managed Transaction ROLLED BACK .........");
			} catch (Exception rollbackExp) {
			  outputErrorMessage("Transaction Rolled Back => Reason: " + rollbackExp);
			} 
			throw e;
		}
		return preparedItem;
	}//end of cookItem

	  // Demostration of WEBLOGIC'S CUSTOM API
	  private void outputErrorMessage(String errorDesc) {
		  String errorMessageToBeDisplayed = "";
    	  String sep = System.getProperty( "line.separator" );

		  weblogic.transaction.Transaction tx = weblogic.transaction.TxHelper.getTransaction();
		  String txName = (tx == null) ? "null" : tx.getName();
		  errorMessageToBeDisplayed = "Transaction Name: " + txName+ sep;

		  errorMessageToBeDisplayed = errorMessageToBeDisplayed + "Error Desc => " + errorDesc + sep;

		  System.out.println( errorMessageToBeDisplayed);
	  } 




}
