package com.sams.learnweblogic7.ejb.cmp;

import java.util.*;
import java.rmi.RemoteException;
import java.util.Properties;

import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.naming.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.ejb.*;

public class WaiterBean implements SessionBean
{
	//defining private and public variables, for the list of items ordered by client, and items cooked by chef
	private Vector orderedItemsList = new Vector();
	private Vector cookedItems = new Vector();
	private static final String CHEFHOME_JNDI_NAME = "Chef_Home";
	private static final boolean ENABLE_WEBLOGIC_LOGGING = true;
    private SessionContext ctx;


	// Using WebLogic's log service to log events
	private void webLogicLog(String logMessage) {
		if (ENABLE_WEBLOGIC_LOGGING) 
			System.out.println(logMessage);
	} //end of webLogicLog


	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Used when create is called on the remote interface
	 */
	public void ejbCreate(){
		webLogicLog("Method ejbCreate has been called");
	}
	
	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Presently not being used
	 */
	public void ejbActivate() {
		webLogicLog("Method ejbActivate has been called");
	}//end of ejbActivate

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Presently not being used
	 */
	public void ejbRemove() {
		webLogicLog("Method ejbRemove has been called");
	}//end of ejbRemove

	/**
	 * As per EJB Specification's to be implemented by bean,
	 * Presently not being used
	 */
	public void ejbPassivate() {
		webLogicLog("Method ejbPassivate has been called");
	}//end of ejbPassivate


	/**
	 * Sets the session context.
	 *
	 * @param ctx SessionContext Context for session
	 */
	public void setSessionContext(SessionContext ctx) {
		webLogicLog("Method setSessionContext has been called");
		this.ctx = ctx;
	}//end of setSessionContext


	//method called by client to get a list of Menu items.
	public MenuList getMenu(){
		return MenuList.getMenu();
	}
	
	//method called by client to get the order. 
	//The client gets the menu item id from the menu list, and also specifies the number of items.
	public void getOrder(MenuItem orderedItem, int quantity){
		System.out.println("Item ordered : "+orderedItem.getItemDesc());
		System.out.println("Item qty ordered : "+quantity);
		orderedItemsList.addElement(new Order(orderedItem, quantity));	
	}
	
	//the client calls this method to confirm his order. When he does this, the waiter requests for 
	//a chef available from the chef pool, and sends the item to be prepared to the chef.
	//The chef returns the cooked item. The waiter then collects all the cooked items into his tray (the global vector),
	//returns the cook into the cook pool.
	public void confirmOrder(){
		CookedItemImpl currentCookedItem;
		MenuItem menuItemToBeProcessed;
		Enumeration orderedItemsEnum = orderedItemsList.elements();
		while(orderedItemsEnum.hasMoreElements())
		{
			try{
				ChefInterface currentChef = getChefFromChefPool();
//				menuItemToBeProcessed = (Order)orderedItemsEnum.nextElement()).getItem();
				currentCookedItem = currentChef.cookItem((Order)orderedItemsEnum.nextElement());
				cookedItems.addElement(currentCookedItem);
			}
			catch(Exception e){
				e.printStackTrace();
			}
		}
	}
	
	//called by the waiter to get a chef from the chef pool
	public ChefInterface getChefFromChefPool(){
		ChefInterface chef=null;
		try{
			ChefHome chefHome = lookupHome();
			chef = (ChefInterface)PortableRemoteObject.narrow(chefHome.create(), ChefInterface.class);
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return chef;
	}
	
	/**
   * Lookup the EJBs home in the JNDI tree
   */
	private ChefHome lookupHome() throws NamingException
	{
	// Lookup the beans home using JNDI
	//(Properties can be used instead of Hashtable, to keep the code compatible with jdk1.1 also.)
	 try{
		Hashtable h = new Hashtable();
		h.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
		h.put(Context.PROVIDER_URL, "t3://localhost:7001");
		Context ctx = new InitialContext(h);
		System.out.println("Initial context to look up chef home created "+ctx);
		Object home = ctx.lookup(CHEFHOME_JNDI_NAME);
		System.out.println("Lookup performed");
		return (ChefHome)PortableRemoteObject.narrow(home, ChefHome.class);
	 } 
	 catch (NamingException namingException) {
		System.out.println("Unable to lookup EJBHome ChefHome.");
		System.out.println("Check JNDI name "+CHEFHOME_JNDI_NAME+" on the WebLogic server");
		throw namingException;
	 }
	}

		
	//performed by the waiter after all the items are cooked, and collected in the vector, and then serves the client.
	public Vector serveCookedItem(){
		return cookedItems;		
	}
	
	//called by the client, to get his check.
	public float getCheck(){
		float checkSum = 0;
		Enumeration orderedItemsEnum = orderedItemsList.elements();
		while(orderedItemsEnum.hasMoreElements())
		{
			Order individualCurrOrder = (Order)orderedItemsEnum.nextElement();
			checkSum += (individualCurrOrder.getItem()).getItemPrice()*individualCurrOrder.getQty();
		}
		return checkSum;
	}
};