import com.sams.learnweblogic7.ejb.jms.*;

import java.rmi.RemoteException;
import java.util.Properties;

import javax.naming.*;
import java.util.*;
import java.io.*;
import java.text.*;
import javax.rmi.PortableRemoteObject;

public class Customer{
	private static final String WAITERHOME_JNDI_NAME = "Waiter_Home";
	private static final String DESIGN_DASH = "-------------------------------------------------------------------";
	
	/**
	 * This is a standalone client. 
   * We run this example from the command line.
   */
  public static void main(String[] args) {
	  
	 /**
	 * create a context object. First create env Hashtable and pass the hashtable as a parameter to the constructor.
	 **/
	  
	  Context ctx = null;
      try {
        Hashtable env = new Hashtable();
        env.put(
          Context.INITIAL_CONTEXT_FACTORY,
          "weblogic.jndi.WLInitialContextFactory"
        );

        ctx = new InitialContext(env);
        System.out.println("Initial context created");

				Hashtable restaurantEnv = new Hashtable();
				restaurantEnv.put(Context.INITIAL_CONTEXT_FACTORY,"com.sun.jndi.fscontext.RefFSContextFactory");
				restaurantEnv.put(Context.PROVIDER_URL,"file:/d:/lakshmi/book");
				Context initCtx = new InitialContext(restaurantEnv);
				System.out.println("Created new initial context");
				try{
					File restPropsFile = (File)initCtx.lookup("code" + File.separator + "JMS" + File.separator + "Client" + File.separator + "RestaurantProps.prop");
					FileInputStream restPropsFileInputStream = new FileInputStream(restPropsFile);
					Properties restaurantProps = new Properties();
					restaurantProps.load(restPropsFileInputStream);
					//decided to go to the restaurant - "The taste of Sams"
					String restaurantCity = restaurantProps.getProperty("The_Taste_Of_Sams_City");
					System.out.println(restaurantCity);
					String restaurantState = restaurantProps.getProperty("The_Taste_Of_Sams_State");
					System.out.println(restaurantState);
					String restaurantType = restaurantProps.getProperty("The_Taste_Of_Sams_Type");
					System.out.println(restaurantType);
				}
				catch(Exception e){
					e.printStackTrace();
				}

				/**
					* Since the client enters the restaurant, to enter, create a restaurant object.
					* then enter the restaurant by calling the enterRestaurant method.
					* Perform a context lookup for the Waiter Object.
					* then perform a lookup for the waiter object.
				**/

					try {
					Restaurant samsRestaurant = new Restaurant();
					String welcomeString = samsRestaurant.enterRestaurant();
					System.out.println(DESIGN_DASH);
					System.out.println(welcomeString);

					//lookup for waiter ejb object
					WaiterInterface thisWaiterObj = null;
					try{
						WaiterHome waiterHome = lookUpHome();
						thisWaiterObj = (WaiterInterface)PortableRemoteObject.narrow(waiterHome.create(),WaiterInterface.class);
					}
					catch(Exception e){
						e.printStackTrace();
					}

					//once, the waiter object is obtained, request the waiter to get the menu list.
					MenuList myMenuList = thisWaiterObj.getMenu();
					System.out.println(DESIGN_DASH);
					System.out.println("Menulist returned");

					//ordering items 1, 2 and 3. To do that, picking up the items from the menulist first.
					MenuItem orderedItem1 = (MenuItem)myMenuList.get("1");
					MenuItem orderedItem2 = (MenuItem)myMenuList.get("2");
					MenuItem orderedItem3 = (MenuItem)myMenuList.get("3");

					thisWaiterObj.getOrder(orderedItem1, 1);
					thisWaiterObj.getOrder(orderedItem2, 2);
					thisWaiterObj.getOrder(orderedItem3, 3);

					//confirming the order
					thisWaiterObj.confirmOrder();

					//once the waiter has confirmed the order, the waiter serves the items.
					Vector cookedItems = thisWaiterObj.serveCookedItem();
					System.out.println("In Customer, got cookedItems object"+cookedItems);
					Enumeration cookedItemsEnum=cookedItems.elements();
					while(cookedItemsEnum.hasMoreElements()){
						CookedItemImpl forthisenum=(CookedItemImpl)cookedItemsEnum.nextElement();
						System.out.println("Cooked items enum");
						System.out.println("Item Name :"+forthisenum.getItemName());
					}
					System.out.println(DESIGN_DASH);
					System.out.println("The served items are:"+cookedItems.size());
					Enumeration servedItemsEnum = cookedItems.elements();
					while(servedItemsEnum.hasMoreElements()){
						CookedItemImpl servedItem = (CookedItemImpl)servedItemsEnum.nextElement();
						System.out.println(DESIGN_DASH);
						System.out.println("Item Name :"+servedItem.getItemName());
						System.out.println("Serving Size :"+servedItem.getServingSize());
						System.out.println("Taste of Item :"+servedItem.getTaste());
						System.out.println("Ingredients of Item :"+servedItem.getIngredients());
						System.out.println("Temperature of Item :"+servedItem.getTemperature());
					}

					//After serving, the client requests for the check.
					float bill = thisWaiterObj.getCheck();
					System.out.println("Your bill is $"+bill);
					}
					catch (Exception e) {
						System.out.println(e.toString());
					}
	    }
		catch (Exception e) {
		System.out.println(e.toString());
		}
		finally {
			if (ctx != null) {
				try {
					ctx.close();
				} 
				catch (NamingException e) {
					System.out.println("Failed to close context due to: " + e);
				}
			}
		}
	}//end of main
	/**
	* Lookup the EJBs home in the JNDI tree
	*/
	public static WaiterHome lookUpHome() throws NamingException
	{
	// Lookup the waiter home bean home using weblogic JNDI
	//(Properties can be used instead of Hashtable, to keep the code compatible with jdk1.1 also.)
	 try{
		Hashtable h = new Hashtable();
		h.put(Context.INITIAL_CONTEXT_FACTORY,"weblogic.jndi.WLInitialContextFactory");
		h.put(Context.PROVIDER_URL, "t3://localhost:7001");
		Context ctx = new InitialContext(h);
		System.out.println("Initial context to look up waiter home created"+ctx);
		Object home = ctx.lookup(WAITERHOME_JNDI_NAME);
		System.out.println("Lookup performed"+home);
		if(home==null){
			System.out.println("Waiter home is null.");
		}
		return (WaiterHome)PortableRemoteObject.narrow(home, WaiterHome.class);
	 } 
	 catch (NamingException namingException) {
		System.out.println("Unable to lookup EJBHome WaiterHome.");
		System.out.println("Check JNDI name "+WAITERHOME_JNDI_NAME+" on the WebLogic server");
		throw namingException;
	 }
	}

}//end of WebLogicContextClient