package com.sams.learnweblogic7.ejb.jms;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collection;
import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.NoSuchEntityException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

public class ItemBean implements EntityBean {
    private EntityContext ctx;
	public int itemId;
	public String itemDesc;
	public String itemTemperature;
	public String itemTaste;
	public String itemIngredients;
	public String itemServingSize;
  
	public void setEntityContext(EntityContext ctx) {
		System.out.println("setEntityContext called "+System.currentTimeMillis());
		this.ctx = ctx;
	}

	public void unsetEntityContext() {
		System.out.println("ItemBean.unsetEntityContext"+System.currentTimeMillis());
		this.ctx = null;
	}

  /**
   * This method is required by the EJB Specification,
   * but is not used by this example.
   * 
   */
  public void ejbActivate() {
      System.out.println("ItemBean.ejbActivate"+System.currentTimeMillis());
  }

  /**
   * This method is required by the EJB Specification,
   * but is not used by this example.
   * 
   */
  public void ejbPassivate() {
    System.out.println("ItemBean.ejbPassivate"+System.currentTimeMillis());
  }

  public void ejbLoad() {
    System.out.println("ItemBean.ejbLoad"+System.currentTimeMillis());
	Connection conn = null;
	PreparedStatement prepStmt = null;
	
	try{
		conn = getConnection();
		prepStmt = conn.prepareStatement("Select item_desc,item_taste,item_temperature,item_ingredients,item_serving_size from item where item_id=?");
		ItemPK key = (ItemPK)ctx.getPrimaryKey();
		itemId = key.itemId;
		prepStmt.setInt(1, itemId);
        ResultSet rs = prepStmt.executeQuery();
		while(rs.next()){
			  itemDesc = rs.getString("item_desc");
			  itemTaste = rs.getString("item_taste");
			  itemTemperature = rs.getString("item_temperature");
			  itemIngredients = rs.getString("item_ingredients");
			  itemServingSize = rs.getString("item_serving_size");
	    } 
		System.out.println("End of ejb load"+System.currentTimeMillis());
	}
	catch(Exception e){
		System.out.println("Error occured in ejb Load");
		e.printStackTrace();
		throw new EJBException("SQL update failed. ejbLoad( ) failed");
	}
	finally{
		try{
			if(conn != null){
				conn.close();
				System.out.println("closed connection in ejb load"+System.currentTimeMillis());
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
  }

  /**
   * Sets the EJBean's modified flag to false.
   * set to false to "reset" the variable for the next transaction.
   * 
   */
  public void ejbStore() {
    System.out.println("ItemBean.ejbStore"+System.currentTimeMillis());
	Connection conn = null;
	PreparedStatement prepStmt = null;
	
	try{
		conn = getConnection();
		prepStmt = conn.prepareStatement("Update Item set item_desc = ?, item_taste = ?, item_temperature = ?, item_ingredients = ?, item_serving_size = ? where item_id=?");
		prepStmt.setString(1, itemDesc);
		prepStmt.setString(2, itemTaste);
		prepStmt.setString(3, itemTemperature);
		prepStmt.setString(4, itemIngredients);
		prepStmt.setString(5, itemServingSize);
		prepStmt.setInt(6, itemId);
		if(prepStmt.executeUpdate() != 1)
            throw new EJBException("SQL update failed. ejbStore( ) failed");
		System.out.println("End of ejb Store"+System.currentTimeMillis());
	}
	catch(Exception e){
		System.out.println("Error in EJB Store method");
		e.printStackTrace();
	}
	finally{
		try{
			if(conn != null){
				conn.close();
				System.out.println("connection closed in ejb store"+System.currentTimeMillis());
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
  }

  /**
   * This method is required by the EJB Specification,
   * but is not used by this example.
   * 
   * @exception               javax.ejb.RemoveException
   *                          if the EJBean does not allow removing the EJBean
   */
  public void ejbRemove()
    throws RemoveException
  {
    System.out.println("ItemBean.ejbRemove "+System.currentTimeMillis());
	Connection conn = null;
	PreparedStatement prepStmt = null;
	
	try{
		conn = getConnection();
		prepStmt = conn.prepareStatement("Delete from Item where item_id = ?");
		prepStmt.setInt(1, itemId);
		prepStmt.executeUpdate();
		System.out.println("end of ejb Remove"+System.currentTimeMillis());
	}
	catch(Exception e){
		System.out.println("Error in ejb Remove");
		e.printStackTrace();
	}
	finally{
		try{
			if(conn != null){
				conn.close();
				System.out.println("connection closed in ejb remove"+System.currentTimeMillis());
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}

  }

  /**
   * This method corresponds to the create method in the home interface
   * "ItemHome.java". 
   * The parameter sets of the two methods are identical.  When the client calls
   * <code>ItemHome.create()</code>, the container (which in WebLogic EJB is
   * also the home) allocates an instance of this EJBean and
   * calls <code>ItemHome.ejbCreate()</code>.
   * <p>
   * For container-managed persistence, <code>ejbCreate()</code> returns 
   * a null, unlike the case of bean-managed
   * persistence, where it returns a primary key.
   *
   * @param itemId				int itemId, 
   * @param temperature			String temperature
   * @param taste			    String taste
   * @param ingredients         String ingredients
   * @param servingSize			String servingSize
   * @exception               javax.ejb.CreateException
   *                          if there is a problem creating the bean
   */
  public ItemPK ejbCreate(int itemId, String temperature, String description, String taste, String ingredients, String servingSize) 
    throws CreateException
  {
    System.out.println("Inside ejb Create"+System.currentTimeMillis());
	
	setItemId(itemId);
	setItemTemperature(temperature);
	setItemDesc(description);
	setItemTaste(taste);
	setItemIngredients(ingredients);
	setItemServingSize(servingSize);
	
	Connection conn = null;
	PreparedStatement prepStmt = null;
	ItemPK key = null;

	try{
		conn = getConnection();
		prepStmt = conn.prepareStatement("Insert into Item (item_id, item_temperature, item_desc, item_taste, item_ingredients, item_serving_size)Values (?,?,?,?,?,?)");
		prepStmt.setInt(1,itemId);
		prepStmt.setString(2,itemTemperature);
		prepStmt.setString(3,itemDesc);
		prepStmt.setString(3,itemTaste);
		prepStmt.setString(4,itemIngredients);
		prepStmt.setString(5,itemServingSize);
		
		key = new ItemPK(itemId);
		System.out.println("end of ejb create"+System.currentTimeMillis());
	}
	catch(Exception e){
		System.out.println("Error in ejbCreate");
		e.printStackTrace();
	}
	finally{
		if(conn!=null){
			try{
				conn.close();		
				System.out.println("Connection closed in ejb create"+System.currentTimeMillis());
			}
			catch(Exception e){
				System.out.println("Error while closing connection in ejbCreate");
			}
		}
	}
	return key;
  }

  /**
   * This method is required by the EJB Specification,
   * but is not used by this example.
   *
   * @param itemId				int itemId, 
   * @param temperature			String temperature
   * @param taste			    String taste
   * @param ingredients         String ingredients
   */
  public void ejbPostCreate(int itemId, String temperature, String description, String taste, String ingredients, String servingSize) 
  {
    System.out.println("ItemBean.ejbPostCreate (" + itemId + ")"+System.currentTimeMillis());
  }

  /**
   * Accessor mutator methods for each of the variables of the bean.
   */
	public int getItemId(){
		return this.itemId;
	}

	public void setItemId(int itemId){
		this.itemId = itemId;
	}

	public String getItemDesc(){
		return this.itemDesc;
	}

	public void setItemDesc(String itemDescription){
		this.itemDesc = itemDescription;
	}

    public String getItemIngredients(){
		return this.itemIngredients;
	}

	public void setItemIngredients(String itemIngredients){
		this.itemIngredients = itemIngredients;
	}

	public String getItemTemperature(){
		return this.itemTemperature;		
	}
	
	public void setItemTemperature(String itemTemperature){
		this.itemTemperature = itemTemperature;
	}

	public String getItemTaste(){
		return this.itemTaste;
	}

	public void setItemTaste(String itemTaste){
		this.itemTaste = itemTaste;
	}

	public String getItemServingSize(){
		return this.itemServingSize;
	}

	public void setItemServingSize(String itemServingSize){
		this.itemServingSize = itemServingSize;
	}

	/**
	*	finder methods:
	*	find by primary key and findIngredientsByItemName(String itemName) 
	*/
	public ItemPK ejbFindByPrimaryKey(ItemPK primaryKey) 
    throws FinderException, RemoteException{
	  System.out.println("ItemBean.ejbFindByPrimaryKey"+System.currentTimeMillis());
	  Connection conn = null;
	  PreparedStatement prepStmt = null;
		
	  try{
	    	conn = getConnection();
			prepStmt = conn.prepareStatement("Select item_id FROM item where item_id=?");
			itemId = primaryKey.itemId;
			prepStmt.setInt(1, itemId);
			ResultSet rs = prepStmt.executeQuery();
			while(rs.next()){
				  itemId = rs.getInt("item_id");
			} 
			System.out.println("end of find by primary key"+System.currentTimeMillis());
		}
		catch(Exception e){
			System.out.println("Error occured in ejbFindByPrimaryKey");
			e.printStackTrace();
			throw new EJBException("SQL update failed. ejbFindByPrimaryKey( ) failed");
		}
		finally{
			try{
				if(conn != null){
					conn.close();
					System.out.println("closed connection of ejb find by primary key"+System.currentTimeMillis());
				}
			}
			catch(Exception e){
				e.printStackTrace();
			}
		}
		return primaryKey;
	  }

	  /**
	   * Gets current connection to the connection pool.
	   *
	   * @return                  Connection
	   * @exception               javax.ejb.EJBException
	   *                          if there is a communications or systems failure
	   */
	  private Connection getConnection()
		throws SQLException
	  {
		InitialContext initCtx = null;
		try {
		  initCtx = new InitialContext();
		  DataSource ds = (javax.sql.DataSource)
			initCtx.lookup("jndi-name-bmp-datasource");
		  System.out.println("in get connection");
		  return ds.getConnection();
		} catch(NamingException ne) {
		  System.out.println("Failed to lookup JDBC Datasource. Please double check that");
		  System.out.println("the JNDI name defined in the resource-description of the ");
		  System.out.println("EJB's weblogic-ejb-jar.xml file is the same as the JNDI name ");
		  System.out.println("for the Datasource defined in your config.xml.");
		  throw new EJBException(ne);
		} finally {
		  try {
			if(initCtx != null) initCtx.close();
			System.out.println("closing connection in get connection");
		  } catch(NamingException ne) {
			System.out.println("Error closing context: " + ne);
			throw new EJBException(ne);
		  }
		}
	  }
}