package com.sams.learnweblogic7.ejb.jms;

import java.io.*;
import java.util.*;
import javax.transaction.*;
import javax.naming.*;
import javax.jms.*;

public class SendMessagesToQueue 
{
	//Defining context factory. This is the default WebLogic's context factory. This class can be found in 
	//the weblogic.jar package located in the C:\bea\weblogic700\server\lib directory.
	public static final String WEBLOGIC_JNDI_FACTORY = "weblogic.jndi.WLInitialContextFactory";

	//defining the name of the JMS connection factory which contains our queues.
	//this has to be configured via the admin console.
	public static final String JMS_FACTORY = "learnweblogic7.jms.QueueConnectionFactory";

	//defining the name of the queue to be used for sending OrderedItem Messages.
	public static final String ORDERED_ITEMS_QUEUE = "learnweblogic7.jms.orderedItemsQueue";

	private QueueConnectionFactory qConnFactory;
	private QueueConnection qConn;
	private QueueSession qSession;
	private QueueSender qSender;
	private Queue queue;
	public static ObjectMessage orderedMsg;

	/****
		Method to obtain the initial context of the queue factory 	
	***/
	private static InitialContext getInitialContext(String contextProviderUrl)
       throws NamingException
    {
		Hashtable envHash = new Hashtable();
		envHash.put(Context.INITIAL_CONTEXT_FACTORY, WEBLOGIC_JNDI_FACTORY);
		//contextProviderUrl is "t3://localhost:7001" in case of weblogic
		envHash.put(Context.PROVIDER_URL, contextProviderUrl);
		return new InitialContext(envHash);
    }

	  /**
	   * Creates all the necessary objects for sending
	   * messages to a JMS queue.
	   *
	   * @param ctx JNDI initial context
	   * @param queueName name of queue
	   * @exception NamingException if operation cannot be performed
	   * @exception JMSException if JMS fails to initialize due to internal error
	   */
    public void init(Context ctx, String queueName)
	   throws NamingException, JMSException
    {
	   qConnFactory = (QueueConnectionFactory) ctx.lookup(JMS_FACTORY);
	   qConn = qConnFactory.createQueueConnection();
	   qSession = qConn.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
	   queue = (Queue) ctx.lookup(queueName);
	   qSender = qSession.createSender(queue);
	   orderedMsg = qSession.createObjectMessage();
	   qConn.start();
    }

	public static void readAndSend(SendMessagesToQueue itemSender, CookedItemImpl cookedItem)
       throws IOException, JMSException
    {
		itemSender.send(cookedItem);
		System.out.println("Sent Message "+cookedItem);
	}

	 /**
	   * Send a message to a JMS queue using this method.
	   *
	   * @params message  message to be sent
	   * @exception JMSException if JMS fails to send message due to internal error
	   */
	   //public void send(ObjectMessage message)
	  public void send(CookedItemImpl cookedItem)
		   throws JMSException
	  {
		orderedMsg.setObject(cookedItem);
		qSender.send(orderedMsg);
	  }

	  /**
	   * Closes JMS objects.
	   * @exception JMSException if JMS fails to close objects due to internal error
	   */
	  public void close()
		   throws JMSException
	  {
		qSender.close();
		qSession.close();
		qConn.close();
	  }

	
}
